#include <iostream>
#include <string>
#include <stdlib.h>
#include <sstream>
#include <fstream>
#include "monge.h"
#include <glpk.h>             /* GNU GLPK linear/mixed integer solver */



//---------------------------------------------------------------------
monge::monge (expression mu , expression nu, expression cost)
{
  f= mu;
  g = nu;
  cout = cost;
}

void monge::construit_marginale(int nb)
{

  g_j.clear();
  f_i.clear();

   int nb2 = 300;

   domain domX;
   domX["x"] = interval(0,1);
   mesh mX(domX,nb) ;

   domain domY;
   domY["y"] = interval(0,1);
   mesh mY(domY,nb) ;

   int index_i = 0;
   int index_j = 0;

   mesh::iterator i;
   for ( i = mX.begin(); i!=mX.end();++i)
      {
	domain region_i =  *i;
	mesh m_region_i(region_i,nb2) ;
	interval f_i_temp = interval(0,0);
	mesh::iterator it_i;

       	for ( it_i = m_region_i.begin(); it_i!=m_region_i.end();++it_i)
	  {
	    domain region_i_k =  *it_i;
	    f_i_temp = f_i_temp + f.Evalue((region_i_k)) * (region_i_k).width();
 	  }
	f_i.push_back(f_i_temp);
	index_i++;
      }

   mesh::iterator j;
   for ( j = mY.begin(); j!=mY.end();++j)
      {
	domain region_j =  *j;
	mesh m_region_j(region_j,nb2) ;
	interval g_j_temp = interval(0,0);
	mesh::iterator it_j;

       	for ( it_j = m_region_j.begin(); it_j!=m_region_j.end();++it_j)
	  {
	    domain region_j_k =  *it_j;
	    g_j_temp = g_j_temp + g.Evalue((region_j_k)) * (region_j_k).width();
 	  }
	g_j.push_back(g_j_temp) ;
	index_j++;
      }
}



void monge::construit_cout(int nb)
{

  cout_ij.clear();
   int variable_index = 0;

   mesh::iterator i,j;

   domain domX;
   domX["x"] = interval(0,1);
   mesh mX(domX,nb) ;

   domain domY;
   domY["y"] = interval(0,1);
   mesh mY(domY,nb) ;

   for ( i = mX.begin(); i!=mX.end();++i)
      {
	vector < interval > cout_j; 
	for ( j = mY.begin(); j!=mY.end();++j)
	  {
	    domain region_i =  *i;
	    domain region_j =  *j;
	   
	    domain region;
	    region = region_i.direct_sum(region_j);
	    cout_j.push_back(cout.Evalue((region)));
	  }
	cout_ij.push_back(cout_j);
      }
}

double monge::dual(int nb)
{


  // petite correction pour prendre en compte le fait que les deux masses à transporter soient égales 
  double MU = 0;
  for ( int i = 1; i<=nb;i++)
    {
      MU =  MU + f_i[i-1].sup();
    }

  std::cout << MU << endl;

  double NU = 0;
  for ( int j = 1; j<=nb;j++)
    {
      NU = NU + g_j[j-1].sup();
    }

  std::cout << NU << endl;


  double erreur = NU-MU;


  double MU_corrige = 0;
  vector < double > f_i_corrige; 
  for ( int i = 1; i<=nb;i++)
    {
      double correction =  f_i[i-1].sup() + erreur/nb;
      MU_corrige =  MU_corrige + correction;
      f_i_corrige.push_back( correction );
    }

  std::cout << MU_corrige << endl;

  erreur = NU-MU_corrige;
  std::cout << erreur << endl;










  glp_prob *lp;
  int taille = nb*nb;
  int ia[1+200000], ja[1+200000];
  double ar[1+200000], z, x1, x2;

  lp = glp_create_prob();
  glp_set_prob_name(lp, "dual_relache");
  glp_set_obj_dir(lp, GLP_MAX);


  // nombre de contraintes, leur nom et leur upper bound
  glp_add_rows(lp, nb*nb);

  int contrainte_index = 1;
  for ( int i = 1; i<=nb;i++)
    {        

      for ( int j = 1; j<=nb;j++)
	{
	std::string contrainte_i_j = "contrainte_";
	contrainte_i_j += std::to_string(i);
	contrainte_i_j += "_";
	contrainte_i_j += std::to_string(j);

	glp_set_row_name(lp, contrainte_index, contrainte_i_j.c_str());
	glp_set_row_bnds(lp, contrainte_index, GLP_UP, 0.0 , cout_ij[i-1][j-1].sup() );
	contrainte_index++;
	}
    }


  // nombre de variables, leur nom et leur lower bound et leur coef dans la fonction cout
  int index_variables  = 1;
  glp_add_cols(lp, 2*nb); 
  for ( int i = 1; i<=nb;i++)
    {        
      std::string phi_i = "phi_";
      phi_i += std::to_string(i);
      
      glp_set_col_name(lp, index_variables, phi_i.c_str());
      glp_set_col_bnds(lp, index_variables, GLP_FR, 0.0, 0.0);
      glp_set_obj_coef(lp, index_variables, f_i_corrige[i-1]);
      //      glp_set_obj_coef(lp, index_variables, f_i[i-1].sup());
      index_variables++;
    }

  for ( int j = 1; j<=nb;j++)
    {        
      std::string psi_j = "psi_";
      psi_j += std::to_string(j);
      
      glp_set_col_name(lp, index_variables, psi_j.c_str());
      glp_set_col_bnds(lp, index_variables, GLP_FR, 0.0, 0.0);
      glp_set_obj_coef(lp, index_variables, g_j[j-1].sup());
      index_variables++;
    }



  // sorte de matrices qui compilent les coefficients des contraintes
  int imc  = 1;
  for ( int j = 1; j<=nb;j++)
    {       
        for ( int i = 1; i<=nb;i++)
	  {       
	    ia[imc] = i+nb*(j-1), ja[imc] = j, ar[imc] = 1.0;
	    imc++;	  
	    // std::cout << i+nb*(j-1) << "," << j << endl;	  
	  }
    }

  //   std::cout << "------------------------------------" << endl;

  for ( int alpha =  0; alpha < nb*nb ; alpha  = alpha + nb)
    {       
        for ( int j = 1; j<= nb ;j++)
	  {       
	    ia[imc] = alpha + j , ja[imc] = j + nb, ar[imc] = 1.0;
	    imc++;	  
	    // std::cout <<  alpha + j  << "," << j + nb << endl;	  
	  }
    }

   glp_load_matrix(lp, imc-1 , ia, ja, ar);
   glp_write_lp(lp, NULL , "test_dual.lp");

   glp_simplex(lp, NULL);
   z = glp_get_obj_val(lp);

   printf("z = %f; \n", z);
   
   glp_delete_prob(lp);
   glp_free_env();
   return z;

    	  



}


double monge::primal(int nb)
{
  glp_prob *lp;
  int taille = nb*nb;
  int ia[1+200000], ja[1+200000];
  double ar[1+200000], z, x1, x2;
  /* create problem */
  lp = glp_create_prob();
  glp_set_prob_name(lp, "primal_relache");
  glp_set_obj_dir(lp, GLP_MIN);

  // nombre de contraintes, leur nom et leur upper bound
  glp_add_rows(lp, 2*nb);

  int contrainte_index = 1;
  for ( int i = 1; i<=nb;i++)
    {        
	std::string contrainte_X_i = "contrainte_X_";
	contrainte_X_i += std::to_string(i);
	glp_set_row_name(lp, contrainte_index, contrainte_X_i.c_str());
	glp_set_row_bnds(lp, contrainte_index, GLP_DB, f_i[i-1].inf(), f_i[i-1].sup()+0.000000000001);
	contrainte_index++;
    }

  for ( int j = 1; j<=nb;j++)
    {
	std::string contrainte_Y_j = "contrainte_Y_";
	contrainte_Y_j += std::to_string(j);
	glp_set_row_name(lp, contrainte_index, contrainte_Y_j.c_str());
	glp_set_row_bnds(lp, contrainte_index, GLP_DB, g_j[j-1].inf(), g_j[j-1].sup());
	contrainte_index++;
    }

  // nombre de variables, leur nom et leur lower bound et leur coef dans la fonction cout
  int index_variables  = 1;
  glp_add_cols(lp, taille); 
  for ( int i = 1; i<=nb;i++)
    {        
      for ( int j = 1; j<=nb;j++)
	{
	  std::string var_zij = "z";
	  var_zij += std::to_string(i);
	  var_zij += "_";
	  var_zij += std::to_string(j);

	  glp_set_col_name(lp, index_variables, var_zij.c_str());
	  glp_set_col_bnds(lp, index_variables, GLP_LO, 0.0, 0.0);
	  glp_set_obj_coef(lp, index_variables, cout_ij[i-1][j-1].inf());
	  index_variables++;
	}
    }	  

  // sorte de matrices qui compilent les coefficients des contraintes
  int imc  = 1;
  std::cout << nb << endl;
  
  for ( int i = 1; i<=nb;i++)
    {       
      int inf = (i-1)*nb + 1;
      int sup = (i-1)*nb + nb;
      for ( int j = inf ; j<= sup;j++)
	{       
	  ia[imc] = i, ja[imc] = j, ar[imc] = 1.0;
	  //  std::cout << i << "," << j << endl;
	  imc++;	  
	}
    }

  for ( int i = 1; i<= nb ;i++)
    {       
      for ( int j = i ; j<= taille; j = j + nb)
	{       
	  ia[imc] = i + nb, ja[imc] = j, ar[imc] = 1.0;
	  //  std::cout << i+nb << "," << j << endl;
	  imc++;	  
	}
    }
  /*
   std::cout << "------------------------------------" << endl;
   std::cout << "chargement" << endl;
   std::cout << "------------------------------------" << endl;
  */
   glp_load_matrix(lp, imc-1 , ia, ja, ar);

   /*  std::cout << "------------------------------------" << endl;
   std::cout << "fin " << endl;
   std::cout << "------------------------------------" << endl;

   std::cout << "------------------------------------" << endl;
   std::cout << "résolution" << endl;
   std::cout << "------------------------------------" << endl;
  
   */

  /* solve problem */
   //    glp_exact(lp, NULL);
   glp_simplex(lp, NULL);
   
   glp_write_lp(lp, NULL , "test.lp");


  /* recover and display results */

    z = glp_get_obj_val(lp);

    /*int lala = 1;
        for ( int i = 1; i<= nb ;i++)
      {       
	for ( int j = 1 ; j<= nb; j++)
	  {       
	    double zij = glp_get_col_prim(lp, lala);	   
	    printf("zij = %e \n", zij);
	    lala++;
	  }
      }
    */
  printf("z = %f; \n", z);

  glp_delete_prob(lp);
  glp_free_env();
  return z;
}

